require 'saml2'
module IntegrationPal
  class SamlController < ApplicationController
    class << self
      def parse_meta_file(filename)
        possible_names = ['.erb', ''].map { |ext| filename+ext }
        possible_paths = []

        ['config/saml', 'config'].each do |prefix|
          possible_names.each do |filename|
            possible_paths << Rails.root.join(prefix, filename)
          end
        end

        meta_path = possible_paths.find &:exist?

        meta_file = File.read(meta_path)
        if meta_path.extname == '.erb'
          meta_file = ERB.new(meta_file).result(ERBContext.new.get_binding)
        end
        SAML2::Entity.parse(meta_file)
      end

      def idp_metadata
        @idp_metadata ||= parse_meta_file(IntegrationPal.saml_idp_metadata)
      end

      def sp_metadata
        @sp_metadata ||= parse_meta_file(IntegrationPal.saml_sp_metadata)
      end

      class ERBContext
        def method_missing(m, *args, **kwargs, &blk)
          if m =~ /(_url)$/ && kwargs[:host].nil?
            uri = URI.parse(ENV["SAML_DOMAIN"])
            kwargs[:host] ||= uri.host
            kwargs[:port] ||= uri.port
            kwargs[:protocol] ||= uri.scheme
          end
          [Rails.application.routes.url_helpers, IntegrationPal::Engine.routes.url_helpers].each do |helper|
            return helper.send(m, *args, **kwargs, &blk) if helper.respond_to?(m)
          end
          super
        end

        def get_binding
          binding
        end
      end
    end

    protect_from_forgery except: [:create]
    skip_before_action :authenticate!

    def new
      authn_request = self.class.sp_metadata.initiate_authn_request(self.class.idp_metadata)
      redirect_to SAML2::Bindings::HTTPRedirect.encode(authn_request)
    end

    def create
      response, _relay_state = SAML2::Bindings::HTTP_POST.decode(request.request_parameters)
      unless self.class.sp_metadata.valid_response?(response, self.class.idp_metadata)
        logger.error("Failed to validate SAML response: #{response.errors}")
        raise ActionController::RoutingError.new('Not Found')
      end

      reset_session
      session[:saml_username] = response.assertions.first.subject.name_id.id
      logger.info("Logged in using SAML2 as #{session[:saml_username]}")

      redirect_to root_url
    end

    def logout
      reset_session
    end

    def metadata
      render xml: self.class.sp_metadata.to_xml
    end
  end
end