require 'active_support/concern'

module IntegrationPal
  module Statusable
    extend ActiveSupport::Concern

    PENDING_STATUS = 'pending'.freeze
    COMPLETED_STATUS = 'completed'.freeze
    IN_PROGRESS_STATUS = 'in_progress'.freeze
    FAILED_STATUS = 'failed'.freeze
    STATUSES = [PENDING_STATUS, COMPLETED_STATUS, IN_PROGRESS_STATUS, FAILED_STATUS].freeze

    included do
      validates :status, presence: true, inclusion: { in: STATUSES }

      before_validation :set_status
      def set_status
        self.status ||= PENDING_STATUS
      end

      def fail(exception)
        self.update_attributes(status: FAILED_STATUS, error: exception.message, backtrace: exception.backtrace)
      end

      def complete
        self.update_attributes(status: COMPLETED_STATUS, finished_at: Time.zone.now)
      end

    end

  end
end