require 'active_support/concern'

module IntegrationPal
  module BaseJob
    extend ActiveSupport::Concern

    included do
      attr_accessor :job

      rescue_from(StandardError) do |exception|
        job.fail(exception)
        raise(exception)
      end

      def setup(active_job)
        @job = IntegrationPal::Job.find(active_job.arguments.first)
        @job.start
        @job
      end

      def teardown(job)
        job.complete
      end

      around_perform do |job, block|
        persisted_job = setup(job)
        block.call
        teardown(persisted_job)
      end
    end

  end
end